@@ -9,10 +9,24 @@ module RSpec
 
     # Stores runtime configuration information.
     #
-    # Configuration options are loaded from `~/.rspec`, `.rspec`,
-    # `.rspec-local`, command line switches, and the `SPEC_OPTS` environment
-    # variable (listed in lowest to highest precedence; for example, an option
-    # in `~/.rspec` can be overridden by an option in `.rspec-local`).
+    # Configuration options are loaded from multiple files and joined together
+    # with command-line switches and the `SPEC_OPTS` environment variable.
+    #
+    # Precedence order (where later entries overwrite earlier entries on
+    # conflicts):
+    #
+    #   * Global (`$XDG_CONFIG_HOME/rspec/options`, or `~/.rspec` if it does
+    #     not exist)
+    #   * Project-specific (`./.rspec`)
+    #   * Local (`./.rspec-local`)
+    #   * Command-line options
+    #   * `SPEC_OPTS`
+    #
+    # For example, an option set in the local file will override an option set
+    # in your global file.
+    #
+    # The global, project-specific and local files can all be overridden with a
+    # separate custom file using the --options command-line parameter.
     #
     # @example Standard settings
     #     RSpec.configure do |c|
@@ -89,7 +103,6 @@ module RSpec
 
       # @macro [attach] add_setting
       #   @!attribute [rw] $1
-      #   @!method $1=(value)
       #
       # @macro [attach] define_reader
       #   @!attribute [r] $1
@@ -101,6 +114,7 @@ module RSpec
       #
       # @note Other scripts invoking `rspec` indirectly will ignore this
       #   setting.
+      # @return [String]
       add_read_only_setting :default_path
       def default_path=(path)
         project_source_dirs << path
@@ -110,6 +124,7 @@ module RSpec
       # @macro add_setting
       # Run examples over DRb (default: `false`). RSpec doesn't supply the DRb
       # server, but you can use tools like spork.
+      # @return [Boolean]
       add_setting :drb
 
       # @macro add_setting
@@ -122,6 +137,7 @@ module RSpec
 
       # Indicates if the DSL has been exposed off of modules and `main`.
       # Default: true
+      # @return [Boolean]
       def expose_dsl_globally?
         Core::DSL.exposed_globally?
       end
@@ -198,24 +214,29 @@ module RSpec
 
       # @macro add_setting
       # The exit code to return if there are any failures (default: 1).
+      # @return [Integer]
       add_setting :failure_exit_code
 
       # @macro add_setting
       # Whether or not to fail when there are no RSpec examples (default: false).
+      # @return [Boolean]
       add_setting :fail_if_no_examples
 
       # @macro define_reader
       # Indicates files configured to be required.
+      # @return [Array<String>]
       define_reader :requires
 
       # @macro define_reader
       # Returns dirs that have been prepended to the load path by the `-I`
       # command line option.
+      # @return [Array<String>]
       define_reader :libs
 
       # @macro add_setting
       # Determines where RSpec will send its output.
       # Default: `$stdout`.
+      # @return [IO, String]
       define_reader :output_stream
 
       # Set the output stream for reporter.
@@ -234,6 +255,7 @@ module RSpec
 
       # @macro define_reader
       # Load files matching this pattern (default: `'**{,/*/**}/*_spec.rb'`).
+      # @return [String]
       define_reader :pattern
 
       # Set pattern to match files to load.
@@ -244,6 +266,7 @@ module RSpec
 
       # @macro define_reader
       # Exclude files matching this pattern.
+      # @return [String]
       define_reader :exclude_pattern
 
       # Set pattern to match files to exclude.
@@ -265,6 +288,7 @@ module RSpec
       # @macro add_setting
       # Report the times for the slowest examples (default: `false`).
       # Use this to specify the number of examples to include in the profile.
+      # @return [Boolean]
       add_setting :profile_examples
 
       # @macro add_setting
@@ -275,55 +299,56 @@ module RSpec
       add_setting :run_all_when_everything_filtered
 
       # @macro add_setting
-      # Color to use to indicate success.
-      # @param color [Symbol] defaults to `:green` but can be set to one of the
-      #   following: `[:black, :white, :red, :green, :yellow, :blue, :magenta,
-      #   :cyan]`
+      # Color to use to indicate success.  Defaults to `:green` but can be set
+      # to one of the following: `[:black, :white, :red, :green, :yellow,
+      # :blue, :magenta, :cyan]`
+      # @return [Symbol]
       add_setting :success_color
 
       # @macro add_setting
-      # Color to use to print pending examples.
-      # @param color [Symbol] defaults to `:yellow` but can be set to one of the
-      #   following: `[:black, :white, :red, :green, :yellow, :blue, :magenta,
-      #   :cyan]`
+      # Color to use to print pending examples.  Defaults to `:yellow` but can
+      # be set to one of the following: `[:black, :white, :red, :green,
+      # :yellow, :blue, :magenta, :cyan]`
+      # @return [Symbol]
       add_setting :pending_color
 
       # @macro add_setting
-      # Color to use to indicate failure.
-      # @param color [Symbol] defaults to `:red` but can be set to one of the
-      #   following: `[:black, :white, :red, :green, :yellow, :blue, :magenta,
-      #   :cyan]`
+      # Color to use to indicate failure.  Defaults to `:red` but can be set to
+      # one of the following: `[:black, :white, :red, :green, :yellow, :blue,
+      # :magenta, :cyan]`
+      # @return [Symbol]
       add_setting :failure_color
 
       # @macro add_setting
-      # The default output color.
-      # @param color [Symbol] defaults to `:white` but can be set to one of the
-      #   following: `[:black, :white, :red, :green, :yellow, :blue, :magenta,
-      #   :cyan]`
+      # The default output color. Defaults to `:white` but can be set to one of
+      # the following: `[:black, :white, :red, :green, :yellow, :blue,
+      # :magenta, :cyan]`
+      # @return [Symbol]
       add_setting :default_color
 
       # @macro add_setting
-      # Color used when a pending example is fixed.
-      # @param color [Symbol] defaults to `:blue` but can be set to one of the
-      #   following: `[:black, :white, :red, :green, :yellow, :blue, :magenta,
-      #   :cyan]`
+      # Color used when a pending example is fixed. Defaults to `:blue` but can
+      # be set to one of the following: `[:black, :white, :red, :green,
+      # :yellow, :blue, :magenta, :cyan]`
+      # @return [Symbol]
       add_setting :fixed_color
 
       # @macro add_setting
-      # Color used to print details.
-      # @param color [Symbol] defaults to `:cyan` but can be set to one of the
-      #   following: `[:black, :white, :red, :green, :yellow, :blue, :magenta,
-      #   :cyan]`
+      # Color used to print details.  Defaults to `:cyan` but can be set to one
+      # of the following: `[:black, :white, :red, :green, :yellow, :blue,
+      # :magenta, :cyan]`
+      # @return [Symbol]
       add_setting :detail_color
 
       # @macro add_setting
       # Don't print filter info i.e. "Run options: include {:focus=>true}"
       # (default `false`).
+      # return [Boolean]
       add_setting :silence_filter_announcements
 
-      # Deprecated. This config option was added in RSpec 2 to pave the way
-      # for this being the default behavior in RSpec 3. Now this option is
-      # a no-op.
+      # @deprecated This config option was added in RSpec 2 to pave the way
+      #   for this being the default behavior in RSpec 3. Now this option is
+      #   a no-op.
       def treat_symbols_as_metadata_keys_with_true_values=(_value)
         RSpec.deprecate(
           "RSpec::Core::Configuration#treat_symbols_as_metadata_keys_with_true_values=",
@@ -387,18 +412,53 @@ module RSpec
       end
 
       # Record the start time of the spec suite to measure load time.
+      # return [Time]
       add_setting :start_time
 
       # @macro add_setting
       # Use threadsafe options where available.
       # Currently this will place a mutex around memoized values such as let blocks.
+      # return [Boolean]
       add_setting :threadsafe
 
       # @macro add_setting
       # Maximum count of failed source lines to display in the failure reports.
       # (default `10`).
+      # return [Integer]
       add_setting :max_displayed_failure_line_count
 
+      # Determines which bisect runner implementation gets used to run subsets
+      # of the suite during a bisection. Your choices are:
+      #
+      #   - `:shell`: Performs a spec run by shelling out, booting RSpec and your
+      #     application environment each time. This runner is the most widely
+      #     compatible runner, but is not as fast. On platforms that do not
+      #     support forking, this is the default.
+      #   - `:fork`: Pre-boots RSpec and your application environment in a parent
+      #     process, and then forks a child process for each spec run. This runner
+      #     tends to be significantly faster than the `:shell` runner but cannot
+      #     be used in some situations. On platforms that support forking, this
+      #     is the default. If you use this runner, you should ensure that all
+      #     of your one-time setup logic goes in a `before(:suite)` hook instead
+      #     of getting run at the top-level of a file loaded by `--require`.
+      #
+      # @note This option will only be used by `--bisect` if you set it in a file
+      #   loaded via `--require`.
+      #
+      # @return [Symbol]
+      attr_reader :bisect_runner
+      def bisect_runner=(value)
+        if @bisect_runner_class && value != @bisect_runner
+          raise "`config.bisect_runner = #{value.inspect}` can no longer take " \
+            "effect as the #{@bisect_runner.inspect} bisect runnner is already " \
+            "in use. This config setting must be set in a file loaded by a " \
+            "`--require` option (passed at the CLI or in a `.rspec` file) for " \
+            "it to have any effect."
+        end
+
+        @bisect_runner = value
+      end
+
       # @private
       # @deprecated Use {#color_mode} = :on, instead of {#color} with {#tty}
       add_setting :tty
@@ -411,8 +471,7 @@ module RSpec
       # @private
       attr_reader :backtrace_formatter, :ordering_manager, :loaded_spec_files
 
-      # rubocop:disable Metrics/AbcSize
-      # rubocop:disable Metrics/MethodLength
+      # rubocop:disable Metrics/AbcSize, Metrics/MethodLength
 
       # Build an object to store runtime configuration options and set defaults
       def initialize
@@ -424,6 +483,9 @@ module RSpec
         @extend_modules  = FilterableItemRepository::QueryOptimized.new(:any?)
         @prepend_modules = FilterableItemRepository::QueryOptimized.new(:any?)
 
+        @bisect_runner = RSpec::Support::RubyFeatures.fork_supported? ? :fork : :shell
+        @bisect_runner_class = nil
+
         @before_suite_hooks = []
         @after_suite_hooks  = []
 
@@ -467,8 +529,7 @@ module RSpec
 
         define_built_in_hooks
       end
-      # rubocop:enable Metrics/MethodLength
-      # rubocop:enable Metrics/AbcSize
+      # rubocop:enable Metrics/MethodLength, Metrics/AbcSize
 
       # @private
       #
@@ -549,6 +610,7 @@ module RSpec
       end
 
       # Returns the configured mock framework adapter module.
+      # @return [Symbol]
       def mock_framework
         if @mock_framework.nil?
           begin
@@ -576,12 +638,13 @@ module RSpec
       # To override this behaviour and display a full backtrace, use
       # `--backtrace` on the command line, in a `.rspec` file, or in the
       # `rspec_options` attribute of RSpec's rake task.
+      # @return [Array<Regexp>]
       def backtrace_exclusion_patterns
         @backtrace_formatter.exclusion_patterns
       end
 
       # Set regular expressions used to exclude lines in backtrace.
-      # @param patterns [Regexp] set the backtrace exlusion pattern
+      # @param patterns [Array<Regexp>] set backtrace_formatter exlusion_patterns
       def backtrace_exclusion_patterns=(patterns)
         @backtrace_formatter.exclusion_patterns = patterns
       end
@@ -594,12 +657,13 @@ module RSpec
       # will be included.
       #
       # You can modify the list via the getter, or replace it with the setter.
+      # @return [Array<Regexp>]
       def backtrace_inclusion_patterns
         @backtrace_formatter.inclusion_patterns
       end
 
       # Set regular expressions used to include lines in backtrace.
-      # @attr patterns [Regexp] set backtrace_formatter inclusion_patterns
+      # @attr patterns [Array<Regexp>] set backtrace_formatter inclusion_patterns
       def backtrace_inclusion_patterns=(patterns)
         @backtrace_formatter.inclusion_patterns = patterns
       end
@@ -862,11 +926,11 @@ module RSpec
       # @overload add_formatter(formatter)
       # @overload add_formatter(formatter, output)
       #
-      # @param formatter [Class, String] formatter to use. Can be any of the
+      # @param formatter [Class, String, Object] formatter to use. Can be any of the
       #   string values supported from the CLI (`p`/`progress`,
-      #   `d`/`doc`/`documentation`, `h`/`html`, or `j`/`json`) or any
+      #   `d`/`doc`/`documentation`, `h`/`html`, or `j`/`json`), any
       #   class that implements the formatter protocol and has registered
-      #   itself with RSpec as a formatter.
+      #   itself with RSpec as a formatter, or a formatter instance.
       # @param output [String, IO] where the formatter will write its output.
       #   Can be an IO object or a string path to a file. If not provided,
       #   the configured `output_stream` (`$stdout`, by default) will be used.
@@ -1452,7 +1516,7 @@ module RSpec
       def requires=(paths)
         directories = ['lib', default_path].select { |p| File.directory? p }
         RSpec::Core::RubyProject.add_to_load_path(*directories)
-        paths.each { |path| require path }
+        paths.each { |path| load_file_handling_errors(:require, path) }
         @requires += paths
       end
 
@@ -1493,7 +1557,7 @@ module RSpec
 
         files_to_run.uniq.each do |f|
           file = File.expand_path(f)
-          load_spec_file_handling_errors(file)
+          load_file_handling_errors(:load, file)
           loaded_spec_files << file
         end
 
@@ -1521,8 +1585,6 @@ module RSpec
       end
 
       # @private
-      # @macro [attach] delegate_to_ordering_manager
-      #   @!method $1
       def self.delegate_to_ordering_manager(*methods)
         methods.each do |method|
           define_method method do |*args, &block|
@@ -1531,12 +1593,12 @@ module RSpec
         end
       end
 
-      # @macro delegate_to_ordering_manager
+      # @!method seed=(value)
       #
       # Sets the seed value and sets the default global ordering to random.
       delegate_to_ordering_manager :seed=
 
-      # @macro delegate_to_ordering_manager
+      # @!method seed
       # Seed for random ordering (default: generated randomly each run).
       #
       # When you run specs with `--order random`, RSpec generates a random seed
@@ -1550,7 +1612,7 @@ module RSpec
       # don't accidentally leave the seed encoded.
       delegate_to_ordering_manager :seed
 
-      # @macro delegate_to_ordering_manager
+      # @!method order=(value)
       #
       # Sets the default global ordering strategy. By default this can be one
       # of `:defined`, `:random`, but is customizable through the
@@ -1560,7 +1622,8 @@ module RSpec
       # @see #register_ordering
       delegate_to_ordering_manager :order=
 
-      # @macro delegate_to_ordering_manager
+      # @!method register_ordering(name)
+      #
       # Registers a named ordering strategy that can later be
       # used to order an example group's subgroups by adding
       # `:order => <name>` metadata to the example group.
@@ -1773,7 +1836,7 @@ module RSpec
       #       require 'support/db'
       #     end
       #   end
-      def when_first_matching_example_defined(*filters, &block)
+      def when_first_matching_example_defined(*filters)
         specified_meta = Metadata.build_hash_from(filters, :warn_about_example_group_filtering)
 
         callback = lambda do |example_or_group_meta|
@@ -1784,7 +1847,7 @@ module RSpec
           # Ensure the callback only fires once.
           @derived_metadata_blocks.delete(callback, specified_meta)
 
-          block.call
+          yield
         end
 
         @derived_metadata_blocks.append(callback, specified_meta)
@@ -1810,6 +1873,12 @@ module RSpec
         handle_suite_hook(scope, meta) do
           @before_suite_hooks << Hooks::BeforeHook.new(block, {})
         end || begin
+          # defeat Ruby 2.5 lazy proc allocation to ensure
+          # the methods below are passed the same proc instances
+          # so `Hook` equality is preserved. For more info, see:
+          # https://bugs.ruby-lang.org/issues/14045#note-5
+          block.__id__
+
           add_hook_to_existing_matching_groups(meta, scope) { |g| g.before(scope, *meta, &block) }
           super(scope, *meta, &block)
         end
@@ -1833,6 +1902,12 @@ module RSpec
         handle_suite_hook(scope, meta) do
           @before_suite_hooks.unshift Hooks::BeforeHook.new(block, {})
         end || begin
+          # defeat Ruby 2.5 lazy proc allocation to ensure
+          # the methods below are passed the same proc instances
+          # so `Hook` equality is preserved. For more info, see:
+          # https://bugs.ruby-lang.org/issues/14045#note-5
+          block.__id__
+
           add_hook_to_existing_matching_groups(meta, scope) { |g| g.prepend_before(scope, *meta, &block) }
           super(scope, *meta, &block)
         end
@@ -1851,6 +1926,12 @@ module RSpec
         handle_suite_hook(scope, meta) do
           @after_suite_hooks.unshift Hooks::AfterHook.new(block, {})
         end || begin
+          # defeat Ruby 2.5 lazy proc allocation to ensure
+          # the methods below are passed the same proc instances
+          # so `Hook` equality is preserved. For more info, see:
+          # https://bugs.ruby-lang.org/issues/14045#note-5
+          block.__id__
+
           add_hook_to_existing_matching_groups(meta, scope) { |g| g.after(scope, *meta, &block) }
           super(scope, *meta, &block)
         end
@@ -1874,6 +1955,12 @@ module RSpec
         handle_suite_hook(scope, meta) do
           @after_suite_hooks << Hooks::AfterHook.new(block, {})
         end || begin
+          # defeat Ruby 2.5 lazy proc allocation to ensure
+          # the methods below are passed the same proc instances
+          # so `Hook` equality is preserved. For more info, see:
+          # https://bugs.ruby-lang.org/issues/14045#note-5
+          block.__id__
+
           add_hook_to_existing_matching_groups(meta, scope) { |g| g.append_after(scope, *meta, &block) }
           super(scope, *meta, &block)
         end
@@ -1883,6 +1970,12 @@ module RSpec
       #
       # See {Hooks#around} for full `around` hook docs.
       def around(scope=nil, *meta, &block)
+        # defeat Ruby 2.5 lazy proc allocation to ensure
+        # the methods below are passed the same proc instances
+        # so `Hook` equality is preserved. For more info, see:
+        # https://bugs.ruby-lang.org/issues/14045#note-5
+        block.__id__
+
         add_hook_to_existing_matching_groups(meta, scope) { |g| g.around(scope, *meta, &block) }
         super(scope, *meta, &block)
       end
@@ -1918,10 +2011,27 @@ module RSpec
         @on_example_group_definition_callbacks ||= []
       end
 
+      # @private
+      def bisect_runner_class
+        @bisect_runner_class ||= begin
+          case bisect_runner
+          when :fork
+            RSpec::Support.require_rspec_core 'bisect/fork_runner'
+            Bisect::ForkRunner
+          when :shell
+            RSpec::Support.require_rspec_core 'bisect/shell_runner'
+            Bisect::ShellRunner
+          else
+            raise "Unsupported value for `bisect_runner` (#{bisect_runner.inspect}). " \
+                  "Only `:fork` and `:shell` are supported."
+          end
+        end
+      end
+
     private
 
-      def load_spec_file_handling_errors(file)
-        load file
+      def load_file_handling_errors(method, file)
+        __send__(method, file)
       rescue Support::AllExceptionsExceptOnesWeMustNotRescue => ex
         relative_file = Metadata.relative_path(file)
         reporter.notify_non_example_exception(ex, "An error occurred while loading #{relative_file}.")
